@@ -12,7 +12,6 @@ from trytond.tools import reduce_ids
 from trytond.pyson import Eval, PYSONEncoder, Date
 from trytond.transaction import Transaction
 from trytond.pool import Pool
-from trytond.config import CONFIG
 from trytond.backend import TableHandler
 
 __all__ = ['TypeTemplate', 'Type', 'OpenType', 'AccountTemplate', 'Account',
@@ -48,12 +47,6 @@ class TypeTemplate(ModelSQL, ModelView):
     @classmethod
     def __setup__(cls):
         super(TypeTemplate, cls).__setup__()
-        cls._constraints += [
-            ('check_recursion', 'recursive_types'),
-        ]
-        cls._error_messages.update({
-            'recursive_types': 'You can not create recursive types!',
-        })
         cls._order.insert(0, ('sequence', 'ASC'))
 
     @classmethod
@@ -66,6 +59,11 @@ class TypeTemplate(ModelSQL, ModelView):
         # Migration from 2.4: drop required on sequence
         table.not_null_action('sequence', action='remove')
 
+    @classmethod
+    def validate(cls, records):
+        super(TypeTemplate, cls).validate(records)
+        cls.check_recursion(records, rec_name='name')
+
     @staticmethod
     def default_balance_sheet():
         return False
@@ -114,6 +112,7 @@ class TypeTemplate(ModelSQL, ModelView):
         pool = Pool()
         Type = pool.get('account.account.type')
         Lang = pool.get('ir.lang')
+        Config = pool.get('ir.configuration')
 
         if template2type is None:
             template2type = {}
@@ -123,9 +122,9 @@ class TypeTemplate(ModelSQL, ModelView):
             vals['company'] = company_id
             vals['parent'] = parent_id
 
-            new_type = Type.create(vals)
+            new_type, = Type.create([vals])
 
-            prev_lang = self._context.get('language') or CONFIG['language']
+            prev_lang = self._context.get('language') or Config.get_language()
             prev_data = {}
             for field_name, field in self._fields.iteritems():
                 if getattr(field, 'translate', False):
@@ -187,12 +186,6 @@ class Type(ModelSQL, ModelView):
     @classmethod
     def __setup__(cls):
         super(Type, cls).__setup__()
-        cls._constraints += [
-            ('check_recursion', 'recursive_types'),
-        ]
-        cls._error_messages.update({
-            'recursive_types': 'You can not create recursive types!',
-        })
         cls._order.insert(0, ('sequence', 'ASC'))
 
     @classmethod
@@ -205,6 +198,11 @@ class Type(ModelSQL, ModelView):
         # Migration from 2.4: drop required on sequence
         table.not_null_action('sequence', action='remove')
 
+    @classmethod
+    def validate(cls, types):
+        super(Type, cls).validate(types)
+        cls.check_recursion(types, rec_name='name')
+
     @staticmethod
     def default_balance_sheet():
         return False
@@ -273,7 +271,9 @@ class Type(ModelSQL, ModelView):
         value, used to convert template id into type. The dictionary is filled
         with new types
         '''
-        Lang = Pool().get('ir.lang')
+        pool = Pool()
+        Lang = pool.get('ir.lang')
+        Config = pool.get('ir.configuration')
 
         if template2type is None:
             template2type = {}
@@ -283,7 +283,7 @@ class Type(ModelSQL, ModelView):
             if vals:
                 self.write([self], vals)
 
-            prev_lang = self._context.get('language') or CONFIG['language']
+            prev_lang = self._context.get('language') or Config.get_language()
             prev_data = {}
             for field_name, field in self.template._fields.iteritems():
                 if getattr(field, 'translate', False):
@@ -361,15 +361,14 @@ class AccountTemplate(ModelSQL, ModelView):
     @classmethod
     def __setup__(cls):
         super(AccountTemplate, cls).__setup__()
-        cls._constraints += [
-            ('check_recursion', 'recursive_accounts'),
-        ]
-        cls._error_messages.update({
-            'recursive_accounts': 'You can not create recursive accounts!',
-        })
         cls._order.insert(0, ('code', 'ASC'))
         cls._order.insert(1, ('name', 'ASC'))
 
+    @classmethod
+    def validate(cls, templates):
+        super(AccountTemplate, cls).validate(templates)
+        cls.check_recursion(templates)
+
     @staticmethod
     def default_kind():
         return 'view'
@@ -430,6 +429,7 @@ class AccountTemplate(ModelSQL, ModelView):
         pool = Pool()
         Account = pool.get('account.account')
         Lang = pool.get('ir.lang')
+        Config = pool.get('ir.configuration')
 
         if template2account is None:
             template2account = {}
@@ -444,9 +444,9 @@ class AccountTemplate(ModelSQL, ModelView):
             vals['type'] = (template2type.get(self.type.id) if self.type
                 else None)
 
-            new_account = Account.create(vals)
+            new_account, = Account.create([vals])
 
-            prev_lang = self._context.get('language') or CONFIG['language']
+            prev_lang = self._context.get('language') or Config.get_language()
             prev_data = {}
             for field_name, field in self._fields.iteritems():
                 if getattr(field, 'translate', False):
@@ -458,7 +458,7 @@ class AccountTemplate(ModelSQL, ModelView):
                     template = self.__class__(self.id)
                     data = {}
                     for field_name, field in self._fields.iteritems():
-                        if (getattr(field, 'translate', False) \
+                        if (getattr(field, 'translate', False)
                                 and (getattr(template, field_name) !=
                                     prev_data[field_name])):
                             data[field_name] = getattr(template, field_name)
@@ -498,7 +498,7 @@ class AccountTemplate(ModelSQL, ModelView):
             if self.taxes:
                 Account.write([Account(template2account[self.id])], {
                         'taxes': [
-                            ('add', [template2tax[x.id] for x in self.taxes])],
+                            ('add', template2tax[x.id]) for x in self.taxes],
                         })
             template_done.append(self.id)
 
@@ -521,8 +521,8 @@ class Account(ModelSQL, ModelView):
     currency_digits = fields.Function(fields.Integer('Currency Digits'),
             'get_currency_digits')
     second_currency = fields.Many2One('currency.currency',
-        'Secondary Currency', help='Force all moves for this account \n' \
-            'to have this secondary currency.', ondelete="RESTRICT")
+        'Secondary Currency', help='Force all moves for this account \n'
+        'to have this secondary currency.', ondelete="RESTRICT")
     type = fields.Many2One('account.account.type', 'Type', ondelete="RESTRICT",
         states={
             'invisible': Eval('kind') == 'view',
@@ -546,8 +546,7 @@ class Account(ModelSQL, ModelView):
         digits=(16, Eval('currency_digits', 2)), depends=['currency_digits']),
         'get_credit_debit')
     reconcile = fields.Boolean('Reconcile',
-        help='Allow move lines of this account \n' \
-            'to be reconciled.',
+        help='Allow move lines of this account \nto be reconciled.',
         states={
             'invisible': Eval('kind') == 'view',
             }, depends=['kind'])
@@ -573,21 +572,22 @@ class Account(ModelSQL, ModelView):
     @classmethod
     def __setup__(cls):
         super(Account, cls).__setup__()
-        cls._constraints += [
-            ('check_recursion', 'recursive_accounts'),
-        ]
         cls._error_messages.update({
-            'recursive_accounts': 'You can not create recursive accounts!',
-            'delete_account_containing_move_lines': 'You can not delete ' \
-                    'accounts containing move lines!',
-        })
+                'delete_account_containing_move_lines': ('You can not delete '
+                    'account "%s" because it has move lines.'),
+                })
         cls._sql_error_messages.update({
-            'parent_fkey': 'You can not delete accounts ' \
-                    'that have children!',
-        })
+                'parent_fkey': ('You can not delete accounts that have '
+                    'children.'),
+                })
         cls._order.insert(0, ('code', 'ASC'))
         cls._order.insert(1, ('name', 'ASC'))
 
+    @classmethod
+    def validate(cls, accounts):
+        super(Account, cls).validate(accounts)
+        cls.check_recursion(accounts)
+
     @staticmethod
     def default_left():
         return 0
@@ -636,15 +636,15 @@ class Account(ModelSQL, ModelView):
                 ('parent', 'child_of', [a.id for a in accounts]),
                 ], query_string=True)
         line_query, fiscalyear_ids = MoveLine.query_get()
-        cursor.execute('SELECT a.id, ' \
-                    'SUM((COALESCE(l.debit, 0) - COALESCE(l.credit, 0))) ' \
-                'FROM account_account a ' \
-                    'LEFT JOIN account_move_line l ' \
-                    'ON (a.id = l.account) ' \
-                'WHERE a.kind != \'view\' ' \
-                    'AND a.id IN (' + query_ids + ') ' \
-                    'AND ' + line_query + ' ' \
-                    'AND a.active ' \
+        cursor.execute('SELECT a.id, '
+                    'SUM((COALESCE(l.debit, 0) - COALESCE(l.credit, 0))) '
+                'FROM account_account a '
+                    'LEFT JOIN account_move_line l '
+                    'ON (a.id = l.account) '
+                'WHERE a.kind != \'view\' '
+                    'AND a.id IN (' + query_ids + ') '
+                    'AND ' + line_query + ' '
+                    'AND a.active '
                 'GROUP BY a.id', args_ids)
         account_sum = {}
         for account_id, sum in cursor.fetchall():
@@ -743,16 +743,16 @@ class Account(ModelSQL, ModelView):
         for i in range(0, len(ids), cursor.IN_MAX):
             sub_ids = ids[i:i + cursor.IN_MAX]
             red_sql, red_ids = reduce_ids('a.id', sub_ids)
-            cursor.execute('SELECT a.id, ' + \
+            cursor.execute('SELECT a.id, ' +
                         ','.join('SUM(COALESCE(l.' + name + ', 0))'
-                            for name in names) + ' ' \
-                    'FROM account_account a ' \
-                        'LEFT JOIN account_move_line l ' \
-                        'ON (a.id = l.account) ' \
-                    'WHERE a.kind != \'view\' ' \
-                        'AND ' + red_sql + ' ' \
-                        'AND ' + line_query + ' ' \
-                        'AND a.active ' \
+                            for name in names) + ' '
+                    'FROM account_account a '
+                        'LEFT JOIN account_move_line l '
+                        'ON (a.id = l.account) '
+                    'WHERE a.kind != \'view\' '
+                        'AND ' + red_sql + ' '
+                        'AND ' + line_query + ' '
+                        'AND a.active '
                     'GROUP BY a.id', red_ids)
             for row in cursor.fetchall():
                 account_id = row[0]
@@ -868,10 +868,12 @@ class Account(ModelSQL, ModelView):
         childs = cls.search([
                 ('parent', 'child_of', [a.id for a in accounts]),
                 ])
-        if MoveLine.search([
-                    ('account', 'in', [a.id for a in childs]),
-                    ]):
-            cls.raise_user_error('delete_account_containing_move_lines')
+        lines = MoveLine.search([
+                ('account', 'in', [a.id for a in childs]),
+                ])
+        if lines:
+            cls.raise_user_error('delete_account_containing_move_lines', (
+                    lines[0].account.rec_name,))
         super(Account, cls).delete(accounts)
 
     def update_account(self, template2account=None, template2type=None):
@@ -883,7 +885,9 @@ class Account(ModelSQL, ModelView):
         template2type is a dictionary with type template id as key and type id
         as value, used to convert type template id into type.
         '''
-        Lang = Pool().get('ir.lang')
+        pool = Pool()
+        Lang = pool.get('ir.lang')
+        Config = pool.get('ir.configuration')
 
         if template2account is None:
             template2account = {}
@@ -896,12 +900,12 @@ class Account(ModelSQL, ModelView):
             current_type = self.type.id if self.type else None
             template_type = (template2type.get(self.template.type.id)
                 if self.template.type else None)
-            if current_type != template_type:
+            if current_type != template2type:
                 vals['type'] = template_type
             if vals:
                 self.write([self], vals)
 
-            prev_lang = self._context.get('language') or CONFIG['language']
+            prev_lang = self._context.get('language') or Config.get_language()
             prev_data = {}
             for field_name, field in self.template._fields.iteritems():
                 if getattr(field, 'translate', False):
@@ -1196,7 +1200,6 @@ class GeneralLedger(Report):
                 end_periods, data['posted'])
             for account in (set(accounts) - set(account2lines)):
                 accounts.remove(account)
-            account_id2lines = None
 
         account_id2lines = cls.lines(accounts,
             list(set(end_periods).difference(set(start_periods))),
@@ -1244,10 +1247,15 @@ class GeneralLedger(Report):
                 balance += line.debit - line.credit
                 res[account.id].append({
                         'date': line.date,
+                        'move': line.move.rec_name,
                         'debit': line.debit,
                         'credit': line.credit,
                         'balance': balance,
-                        'name': line.description,
+                        'description': '\n'.join(
+                            (line.move.description or '',
+                                line.description or '')).strip(),
+                        'origin': (line.move.origin.rec_name
+                            if line.move.origin else ''),
                         'state': state_selections.get(line.move.state,
                             line.move.state),
                         })
@@ -1490,7 +1498,7 @@ class OpenBalanceSheet(Wizard):
                 'posted': self.start.posted,
                 'company': company.id,
                 })
-        action['name'] += '%s - %s' % (date, company.name)
+        action['name'] += ' %s - %s' % (date, company.rec_name)
         return action, {}
 
     def transition_open_(self):
@@ -1649,9 +1657,11 @@ class CreateChart(Wizard):
         TaxTemplate = pool.get('account.tax.template')
         TaxRuleTemplate = pool.get('account.tax.rule.template')
         TaxRuleLineTemplate = \
-                pool.get('account.tax.rule.line.template')
+            pool.get('account.tax.rule.line.template')
+        Config = pool.get('ir.configuration')
 
-        with Transaction().set_context(language=CONFIG['language']):
+        with Transaction().set_context(language=Config.get_language(),
+                company=self.account.company.id):
             account_template = self.account.account_template
 
             # Create account types
@@ -1732,12 +1742,12 @@ class CreateChart(Wizard):
             with Transaction().set_user(0):
                 Property.delete(properties)
                 if self.properties.account_receivable:
-                    Property.create({
-                            'field': account_receivable_field.id,
-                            'value': 'account.account,' + \
-                                str(self.properties.account_receivable.id),
-                            'company': self.properties.company.id,
-                            })
+                    Property.create([{
+                                'field': account_receivable_field.id,
+                                'value': str(
+                                    self.properties.account_receivable),
+                                'company': self.properties.company.id,
+                                }])
 
             account_payable_field, = ModelField.search([
                     ('model.model', '=', 'party.party'),
@@ -1751,12 +1761,11 @@ class CreateChart(Wizard):
             with Transaction().set_user(0):
                 Property.delete(properties)
                 if self.properties.account_payable:
-                    Property.create({
-                            'field': account_payable_field.id,
-                            'value': 'account.account,' + \
-                                str(self.properties.account_payable.id),
-                            'company': self.properties.company.id,
-                            })
+                    Property.create([{
+                                'field': account_payable_field.id,
+                                'value': str(self.properties.account_payable),
+                                'company': self.properties.company.id,
+                                }])
         return 'end'
 
 
@@ -1796,7 +1805,7 @@ class UpdateChart(Wizard):
         TaxRuleTemplate = pool.get('account.tax.rule.template')
         TaxRuleLine = pool.get('account.tax.rule.line')
         TaxRuleLineTemplate = \
-                pool.get('account.tax.rule.line.template')
+            pool.get('account.tax.rule.line.template')
 
         account = self.start.account
 
@@ -1963,20 +1972,20 @@ class ThirdPartyBalance(Report):
         else:
             posted_clause = ""
 
-        cursor.execute('SELECT l.party, SUM(l.debit), SUM(l.credit) ' \
-                'FROM account_move_line l ' \
+        cursor.execute('SELECT l.party, SUM(l.debit), SUM(l.credit) '
+                'FROM account_move_line l '
                     'JOIN account_move m ON (l.move = m.id) '
                     'JOIN account_account a ON (l.account = a.id) '
-                'WHERE l.party IS NOT NULL '\
-                    'AND a.active ' \
-                    'AND a.kind IN (\'payable\',\'receivable\') ' \
-                    'AND l.reconciliation IS NULL ' \
-                    'AND a.company = %s ' \
-                    'AND (l.maturity_date <= %s ' \
-                        'OR l.maturity_date IS NULL) '\
-                    'AND ' + line_query + ' ' \
-                    + posted_clause + \
-                'GROUP BY l.party ' \
+                'WHERE l.party IS NOT NULL '
+                    'AND a.active '
+                    'AND a.kind IN (\'payable\',\'receivable\') '
+                    'AND l.reconciliation IS NULL '
+                    'AND a.company = %s '
+                    'AND (l.maturity_date <= %s '
+                        'OR l.maturity_date IS NULL) '
+                    'AND ' + line_query + ' '
+                    + posted_clause +
+                'GROUP BY l.party '
                 'HAVING (SUM(l.debit) != 0 OR SUM(l.credit) != 0)',
                 (data['company'], Date.today()))
 
@@ -2057,7 +2066,8 @@ class OpenAgedBalance(Wizard):
         super(OpenAgedBalance, cls).__setup__()
         cls._error_messages.update({
                 'warning': 'Warning',
-                'term_overlap_desc': 'You cannot define overlapping terms'})
+                'term_overlap_desc': 'You cannot define overlapping terms',
+                })
 
     def do_print_(self, action):
         if not (self.start.term1 < self.start.term2 < self.start.term3):
@@ -2138,8 +2148,8 @@ class AgedBalance(Report):
                 if party in res:
                     res[party][position] = solde
                 else:
-                    res[party] = [(i[0] == position) and solde \
-                            or Decimal("0.0") for i in enumerate(terms)]
+                    res[party] = [(i[0] == position) and solde
+                        or Decimal("0.0") for i in enumerate(terms)]
         parties = Party.search([
             ('id', 'in', [k for k in res.iterkeys()]),
             ])
